#-*-coding: utf-8-*-
from datetime import date
import decimal

def AfiClassExtract(register):
    '''Zwraca w postaci słownika poszczególne dane funduszu.'''
    units = []
    buyprice = []
    buydate = []
    invested = []
    for i in register.subreg:
        units.append(i.units)
        buyprice.append(i.buyprice)
        buydate.append(i.buydate)
        invested.append(i.invested)

    return {'units': units, 'buyprice': buyprice, 'buydate': buydate, 'invested': invested, 'price': register.price, 'date': register.date}
    

class Statistics():
    def __init__(self, units=[], price=decimal.Decimal(0), buyprice=[], buydate=[], date='', invested=[]):
        #give value
        #those with array have values of one transaction
        self.setunits(units)
        self.setprice(price)
        self.setdate(date)
        self.setbuyprice(buyprice)
        self.setbuydate(buydate)
        self.setinvested(invested)

    def __str__(self):
        a = '''Aktualnie (%s):
Liczba j. (%s) Cena (%s) Wartość (%s)\n''' % (self.date, self.getnumberofunits(), self.price, round(self.getcurrentvalue(),2))
        a += '''Wartość zakupu: %s Wartość inwestycji:%s \n\n''' % (round(self.getbuyvalue(),2), round(self.getinvestedvalue(),2))
        a += '''Różnica:
*zmiana wartości od zakupu: %s (%s %%)
*zmiana wartości od inwestycji: %s (%s %%)
*zmiana w stosunku rocznym ważona przez wartość zakupu: %s %%''' % (round(self.getvaluechange(),2), round(self.getpercentagechange(),2), round(self.getinvestedvaluechange(),2), round(self.getinvestedpercentagechange(),2), round(self.getannualpercentageratebybuyvalue(),2))
        return a

#seting value
    def setunits(self, array):
        '''Ilość zakupionych j. dla każdej tranzakcji. W formie tablicy.'''
        #sprawdzamy czy tablica
        if type(array) != type([]): array = [] #zamiana na pustą jeśli nie jest tablicą
        self.units = array

    def setprice(self, price):
        '''Aktulana wycena j.'''
        self.price = price

    def setdate(self, date):
        '''Data aktualnej wyceny j.'''
        self.date = date

    def setbuyprice(self, array):
        '''Cena zakupionych j. dla każdej tranzakcji. W formie tablicy.'''
        #sprawdzamy czy tablica
        if type(array) != type([]): array = [] #zamiana na pustą jeśli nie jest tablicą
        self.buyprice = array

    def setbuydate(self, array):
        '''Data dokonania każdej tranzakcji zakupu j. W formie tablicy.
Ułożone chronologicznie od najstarszego zakupu.'''
        #sprawdzamy czy tablica
        if type(array) != type([]): array = [] #zamiana na pustą jeśli nie jest tablicą
        self.buydate = array

    def setinvested(self, array):
        '''Kwota jaka była zainwenstowana na zakup j. dla każdej tranzakcji.
Jest to kwota bez dopisywanych odsetek. Wkład własny. W formie tablicy.'''
        #sprawdzamy czy tablica
        if type(array) != type([]): array = [] #zamiana na pustą jeśli nie jest tablicą
        self.invested = array #invested - ile pieniędzy zainwestowane, ważne przy subfunduszach

#calculating
    def getnumberofunits(self):
        '''Zwraca ilość j.'''
        units = decimal.Decimal(0)
        for i in range(len(self.units)):
            units = units + self.units[i]
        return units

    def getcurrentvalue(self):
        '''Zwraca wartość aktywów przy bieżącej cenie.'''
        value = self.getnumberofunits() * self.price
        return value
    
    def getbuyvalue(self):
        '''Zwraca wartość zakupu j.'''
        if not len(self.units) == len(self.buyprice): return 'arrays len are not equal'  

        value = decimal.Decimal(0)

        for i in range(len(self.units)):
            value = value + self.units[i] * self.buyprice[i]

        return value
    
    def getinvestedvalue(self):
        '''Zwraca wartość zainwestowanej kwoty (wkładu własnego.'''
        invested = decimal.Decimal(0)

        for i in range(len(self.invested)):
            invested = invested + self.invested[i]

        return invested


    def getvaluechange(self):
        '''Zwraca wartość różnicy pomiędzy obecną wartością a sumą wartości podczas zakupu.'''
        valuechange = self.getcurrentvalue() - self.getbuyvalue()
        return valuechange

    def getinvestedvaluechange(self):
        '''Zwraca wartość różnicy pomiędzy obecną wartością a sumą wartości iwnestycji (wkładu własnego).'''
        investedvaluechange = self.getcurrentvalue() - self.getinvestedvalue()
        return investedvaluechange

    def getpercentagechange(self):
        '''Zwraca stosunek różnicy pomiędzy obecną wartością a sumą wartości podczas zakupu do sumy wartości zakupu.
% = (obecna_wartość - wartość_zakupu) / wartość_zakup'''
        percent = self.getvaluechange() / self.getbuyvalue() * decimal.Decimal(100)
        return percent

    def getinvestedpercentagechange(self):
        '''Zwraca stosunek różnicy pomiędzy obecną wartością a sumą inwestycji do sumy inwestycji (wkładu własnego).
% = (obecna_wartość - inwestycja) / inwestycja'''
        percent = self.getinvestedvaluechange() / self.getinvestedvalue() * decimal.Decimal(100)
        return percent

    def getannualpercentageratebybuyvalue(self):
        '''Zwraca zważoną przez kwotę zakupu średnioroczną stopę procentową.
% = (obecna_wartość - wartość_zakupu) / wartość_zakup * 365 / SUMA(wartość_zakupu_i / wartość_zakupu * dni(data_sprzedaży - data_zakupu))'''
        suma = decimal.Decimal(0)
        buyvalue = self.getbuyvalue()
        for i in range(len(self.units)):
            suma += self.units[i] * self.buyprice[i] / buyvalue * decimal.Decimal((self.date - self.buydate[i]).days)

        return self.getvaluechange() / buyvalue * decimal.Decimal(365) / suma * decimal.Decimal(100)
#taxing
